import logging

import itunespy

logger = logging.getLogger(__name__)


class Bunch:
    def __init__(self, **kwds):
        self.__dict__.update(kwds)


def get_metadata(title: str):
    logger.debug("Getting metadata for %s.", title)
    try:
        # hopefully in the "artist - track" format
        if " - " in title:
            artist_name, track_name = title.split(" - ", 1)
            return Bunch(
                artist_name=artist_name.strip(), track_name=track_name.strip()
            )
        # if not, searching tags on itunes
        else:
            return itunespy.search_track(title)[0]
    except Exception as e:
        logger.error("Error finding tags for %s : %s.", title, e)
        return None



from threading import Thread
import logging
from youtube_dl import YoutubeDL
from tqdm import tqdm
from ypc.tag_utils import get_metadata

logger = logging.getLogger(__name__)


class YdlDownloadThread(Thread):
    def __init__(self, num, rows, only_audio=False):
        Thread.__init__(self)
        self.num = num
        self.rows = rows
        self.only_audio = only_audio
        logger.debug("Init thread %s.", self.num)

    def run(self):
        for index, row in tqdm(
            self.rows.iterrows(),
            dynamic_ncols=True,
            total=self.rows.shape[0],
            position=self.num,
        ):
            logger.debug(
                "Thread %s : Downloading %s - %s.", self.num, index, row["url"]
            )
            try:
                self.ydl_download(row)
            except Exception as e:
                logger.error(
                    "Error downloading %s in thread %s : %s.",
                    row["url"],
                    self.num,
                    e,
                )

    def ydl_download(self, row):  # pragma: no cover
        ydl_opts_base = {
            "logger": MyLogger(),
            "outtmpl": "%(title)s.%(ext)s",
            "prefer_ffmpeg": True,
            "download_archive": "ypc_progress.txt",
        }
        if not row.empty:
            metadata = None
            if "title" in row:
                metadata = get_metadata(row["title"])
            if self.only_audio:
                ydl_opts = {
                    **ydl_opts_base,
                    "writethumbnail": True,
                    "format": "bestaudio/best",
                    "postprocessors": [
                        {
                            "key": "FFmpegExtractAudio",
                            "preferredcodec": "mp3",
                            "preferredquality": "256",
                        },
                        {"key": "EmbedThumbnail"},
                    ],
                }
                if metadata:
                    ydl_opts = {
                        **ydl_opts,
                        "postprocessor_args": [
                            "-metadata",
                            f"artist={metadata.artist_name}",
                            "-metadata",
                            f"title={metadata.track_name}",
                        ],
                    }
            else:
                ydl_opts = {
                    **ydl_opts_base,
                    "format": "bestvideo[height<=1080]+bestaudio/best[height<=1080]",
                    # Would need to recode in mp4 or m4a. Useless for now.
                    # "writethumbnail": True,
                    # "postprocessors": [{"key": "EmbedThumbnail"}],
                }
            with YoutubeDL(ydl_opts) as ydl:
                ydl.extract_info(row["url"], download=True)
        else:
            logger.warning("Url invalid : %s. Skipping.", row["url"])


def dict_is_song(info_dict):
    """ Determine if a dictionary returned by youtube_dl is from a song (and not an album for example). """
    if "full album" in info_dict["title"].lower():
        return False
    if int(info_dict["duration"]) > 7200:
        return False
    return True


def get_ydl_dict(search_term, position):
    ydl_opts = {"logger": MyLogger()}
    with YoutubeDL(ydl_opts) as ydl:
        info_dict = ydl.extract_info(
            f"ytsearch{position}:{search_term}", download=False
        )
    return info_dict["entries"][position - 1]


def get_youtube_url(search_term):
    """ Extract an url for a song. """
    position = 1
    while True:
        try:
            info_dict = get_ydl_dict(search_term, position)
            if dict_is_song(info_dict):
                break
        except Exception as e:
            logger.error(
                "Error extracting youtube search %s, position %s : %s.",
                search_term,
                position,
                e,
            )
            if position > 4:
                # Too many wrong results
                return None
        position += 1
    return info_dict["webpage_url"]


class MyLogger(object):  # pragma: no cover
    def debug(self, msg):
        pass

    def warning(self, msg):
        pass

    def error(self, msg):
        print(msg)



import pytest
import os
from ypc import ydl_utils

SEARCH_FILE = "tests/test_files/search_terms.txt"
SEARCH_TERM = "xtc - making plans for nigel"
SEARCH_TERM_IS_ALBUM = "john coltrane - my favorite things full album"
SEARCH_TERM_IS_TOO_LONG = "crab rave 10 hours"
YOUTUBE_FILE = "tests/test_files/youtube_urls.txt"
YOUTUBE_URL = "https://www.youtube.com/watch?v=WfKhVV-7lxI"


@pytest.mark.skipif(
    "TRAVIS" in os.environ and os.environ["TRAVIS"] == "true",
    reason="doesn't work with Travis",
)
def test_get_youtube_url():
    if not isinstance(ydl_utils.get_youtube_url(SEARCH_TERM), str):
        raise AssertionError()


@pytest.mark.skipif(
    "TRAVIS" in os.environ and os.environ["TRAVIS"] == "true",
    reason="doesn't work with Travis",
)
def test_get_ydl_dict():
    info_dict = ydl_utils.get_ydl_dict(SEARCH_TERM, 1)
    if not isinstance(info_dict, dict):
        raise AssertionError()

    if not ydl_utils.dict_is_song(info_dict):
        raise AssertionError()


@pytest.mark.skipif(
    "TRAVIS" in os.environ and os.environ["TRAVIS"] == "true",
    reason="doesn't work with Travis",
)
def test_dict_is_song():
    if ydl_utils.dict_is_song(ydl_utils.get_ydl_dict(SEARCH_TERM_IS_ALBUM, 1)):
        raise AssertionError()

    if ydl_utils.dict_is_song(
        ydl_utils.get_ydl_dict(SEARCH_TERM_IS_TOO_LONG, 1)
    ):
        raise AssertionError()



import requests
import logging
import pandas as pd

logger = logging.getLogger(__name__)


def get_deezer_playlist_tracks(playlist):
    df = pd.DataFrame()
    playlist = (
        f"https://api.deezer.com/playlist/{playlist.split('/')[-1]}/tracks"
    )
    list_res = []
    logger.debug("Playlist api url : %s", playlist)
    res = requests.get(playlist).json()
    list_res.append(res)
    while "next" in res:
        logger.debug("next tracks in playlist : %s", res["next"])
        res = requests.get(res["next"]).json()
        list_res.append(res)
        if "next" not in res:
            break
    for res in list_res:
        for track in res["data"]:
            artist = track["artist"]["name"]
            title = track["title"]
            df = df.append(
                {
                    "artist": artist,
                    "track_name": title,
                    "title": artist + " - " + title,
                    "playlist_url": playlist,
                },
                ignore_index=True,
            )
    return df


def get_deezer_album_tracks(album):
    df = pd.DataFrame()
    api_url = f"https://api.deezer.com/album/{album.split('/')[-1]}"
    logger.debug("Album api url : %s", api_url)
    res = requests.get(api_url).json()
    logger.debug("Album deezer %s : %s.", album, res)
    for track in res["tracks"]["data"]:
        artist = track["artist"]["name"]
        title = track["title"]
        df = df.append(
            {
                "artist": artist,
                "track_name": title,
                "album": res["title"],
                "album_url": album,
                "title": artist + " - " + title,
            },
            ignore_index=True,
        )
    return df


def get_deezer_songs(terms):
    df = pd.DataFrame()
    for item in terms:
        logger.debug(item)
        # item is album
        if "album" in item:
            df = pd.concat([df, get_deezer_album_tracks(item)], sort=False)
        # item is playlist
        elif "playlist" in item:
            df = pd.concat([df, get_deezer_playlist_tracks(item)], sort=False)
        else:
            logger.error(
                "%s not supported. Please retry with another search.", item
            )
            raise SystemExit
    return df



from ypc import deezer_utils
import pandas as pd
import pytest

DEEZER_FILE = "tests/test_files/deezer_urls.txt"
DEEZER_PLAYLIST_URL = "https://www.deezer.com/fr/playlist/1914526462"
DEEZER_ALBUM_URL = "https://www.deezer.com/fr/album/95467"


def test_get_deezer_playlist_tracks():
    playlist_df = deezer_utils.get_deezer_playlist_tracks(DEEZER_PLAYLIST_URL)
    if not isinstance(playlist_df, pd.DataFrame):
        raise AssertionError()

    with pytest.raises(Exception):
        deezer_utils.get_deezer_playlist_tracks(DEEZER_ALBUM_URL)
        deezer_utils.get_deezer_playlist_tracks(DEEZER_FILE)


def test_get_deezer_album_tracks():
    if not isinstance(
        deezer_utils.get_deezer_album_tracks(DEEZER_ALBUM_URL), pd.DataFrame
    ):
        raise AssertionError()

    with pytest.raises(Exception):
        deezer_utils.get_deezer_album_tracks(DEEZER_PLAYLIST_URL)
        deezer_utils.get_deezer_album_tracks(DEEZER_FILE)


def test_get_deezer_songs():
    if not isinstance(
        deezer_utils.get_deezer_songs([DEEZER_PLAYLIST_URL, DEEZER_ALBUM_URL]),
        pd.DataFrame,
    ):
        raise AssertionError()



from ypc import tag_utils


def test_tag():
    # first method : string matching
    metadata = tag_utils.get_metadata("Pinback - Loro")
    if not metadata.artist_name == "Pinback":
        raise AssertionError()
    if not metadata.track_name == "Loro":
        raise AssertionError()

    # second method : itunes search
    metadata = tag_utils.get_metadata("radiohead in limbo")
    print(metadata)
    if not metadata.artist_name == "Radiohead":
        raise AssertionError()
    if not metadata.track_name == "In Limbo":
        raise AssertionError()


def test_tag_invalid():
    metadata = tag_utils.get_metadata("test metadata invalid")
    if metadata is not None:
        raise AssertionError()



from ypc import spotify_utils
import pandas as pd
import pytest

SPOTIFY_FILE = "tests/test_files/spotify_urls.txt"
SPOTIFY_PLAYLIST_URL = (
    "https://open.spotify.com/playlist/37i9dQZF1DX2sUQwD7tbmL"
)
SPOTIFY_ALBUM_URL = "https://open.spotify.com/album/4FCoFSNIFhK36holxHWCnc"


def test_get_spotify_playlist_tracks(sp):
    if not isinstance(
        spotify_utils.get_spotify_playlist_tracks(sp, SPOTIFY_PLAYLIST_URL),
        pd.DataFrame,
    ):
        raise AssertionError()
    with pytest.raises(Exception):
        spotify_utils.get_spotify_playlist_tracks(sp, SPOTIFY_ALBUM_URL)
        spotify_utils.get_spotify_playlist_tracks(sp, SPOTIFY_FILE)


def test_get_spotify_album_tracks(sp):
    if not isinstance(
        spotify_utils.get_spotify_album_tracks(sp, SPOTIFY_ALBUM_URL),
        pd.DataFrame,
    ):
        raise AssertionError()
    with pytest.raises(Exception):
        spotify_utils.get_spotify_album_tracks(sp, SPOTIFY_PLAYLIST_URL)
        spotify_utils.get_spotify_album_tracks(sp, SPOTIFY_FILE)


def test_get_spotify_songs():
    if not isinstance(
        spotify_utils.get_spotify_songs(
            [SPOTIFY_PLAYLIST_URL, SPOTIFY_ALBUM_URL]
        ),
        pd.DataFrame,
    ):
        raise AssertionError()



import logging
import configparser
import os
import spotipy
import pandas as pd
from spotipy.oauth2 import SpotifyClientCredentials

logger = logging.getLogger(__name__)


def get_spotipy():  # pragma: no cover
    # test if in Travis-CI
    if "TRAVIS" in os.environ and os.environ["TRAVIS"] == "true":
        try:
            client_credentials_manager = SpotifyClientCredentials(
                client_id=os.environ["SPOTIFY_CLIENT_ID"],
                client_secret=os.environ["SPOTIFY_CLIENT_SECRET"],
            )
            return spotipy.Spotify(
                client_credentials_manager=client_credentials_manager
            )
        except Exception as e:
            logger.error(e)
    else:
        # Spotify config file parsing
        user_config_dir = os.path.expanduser("~/.config/ypc/")
        try:
            config = configparser.ConfigParser()
            config.read(user_config_dir + "config.ini")
            spotify_id = config["spotify"]["id"]
            secret = config["spotify"]["secret"]
        except Exception as e:
            logger.error(
                "Error with the config file. Be sure to have a valid ~/.config/ypc/config.ini file if you want to use the spotify playlist extraction features. Error : %s",
                e,
            )
            if not os.path.exists(user_config_dir):
                logger.info(
                    "Configuration folder not found. Creating ~/.config/ypc/."
                )
                os.makedirs(user_config_dir)
            if not os.path.isfile(user_config_dir + "config.ini"):
                sample_config = (
                    "[spotify]\n"
                    "id=spotify_id_here\n"
                    "secret=spotify_secret_here\n"
                )
                with open(
                    user_config_dir + "config.ini", "w", encoding="utf-8"
                ) as f:
                    f.write(sample_config)
                logger.info(
                    "A sample configuration file has been created at ~/.config/ypc/config.ini. Go to https://developer.spotify.com/dashboard/login to create your own spotify application."
                )
            raise SystemExit

        # Spotify API
        client_credentials_manager = SpotifyClientCredentials(
            client_id=spotify_id, client_secret=secret
        )
        sp = spotipy.Spotify(
            client_credentials_manager=client_credentials_manager
        )
        return sp


def get_spotify_playlist_tracks(sp, playlist_id):
    df = pd.DataFrame()
    results = sp.playlist_tracks(playlist_id)
    tracks = results["items"]
    while results["next"]:
        results = sp.next(results)
        tracks.extend(results["items"])
    for song in tracks:
        artist = str(song["track"]["artists"][0]["name"])
        title = str(song["track"]["name"])
        df = df.append(
            {
                "title": artist + " - " + title,
                "playlist_url": playlist_id,
                "track_name": title,
                "artist": artist,
            },
            ignore_index=True,
        )
    return df


def get_spotify_album_tracks(sp, album_id):
    df = pd.DataFrame()
    results = sp.album_tracks(album_id, limit=None)
    logger.debug("Album tracks for %s : %s.", album_id, results)
    for song in results["items"]:
        artists = ",".join([x["name"] for x in song["artists"]])
        title = song["name"]
        track_number = int(song["track_number"])
        df = df.append(
            {
                "title": artists + " - " + title,
                "album_url": album_id,
                "track_name": title,
                "artist": artists,
                "track_number": track_number,
            },
            ignore_index=True,
        )
    return df


def get_spotify_songs(terms):
    sp = get_spotipy()
    df = pd.DataFrame()
    for item in terms:
        logger.debug(item)
        # item is album
        if "album" in item:
            df = pd.concat(
                [df, get_spotify_album_tracks(sp, album_id=item)], sort=False
            )
        # item is playlist
        elif "playlist" in item:
            df = pd.concat(
                [df, get_spotify_playlist_tracks(sp, playlist_id=item),],
                sort=False,
            )
        else:
            logger.error(
                "%s not supported. Please retry with another search.", item
            )
            raise SystemExit
    return df



import setuptools
import ypc

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ypc",
    version=ypc.__version__,
    author="dbeley",
    author_email="dbeley@protonmail.com",
    description="Convert spotify/deezer/text playlists to youtube urls or audio/video files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dbeley/ypc",
    packages=setuptools.find_packages(),
    include_package_data=True,
    entry_points={"console_scripts": ["ypc=ypc.__main__:main"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX :: Linux",
    ],
    install_requires=[
        "requests",
        "spotipy",
        "pandas",
        "beautifulsoup4",
        "youtube-dl",
        "lxml",
        "tqdm",
        "itunespy",
    ],
)



import argparse
import logging
import os
import time
import pandas as pd
import numpy as np
from pathlib import Path
from tqdm import tqdm
from ypc.spotify_utils import get_spotify_songs
from ypc.deezer_utils import get_deezer_songs
from ypc.ydl_utils import YdlDownloadThread, get_youtube_url

logger = logging.getLogger()
FORMAT = "%(levelname)s :: %(message)s"
temps_debut = time.time()


def extract_terms_from_file(file):
    with open(file, "r", encoding="utf-8") as f:
        terms = [line.strip() for line in f]
    return terms


def extract_terms_from_arg(arg):
    return [x.strip() for x in arg.split(",")]


def main_argument_is_youtube(argument):
    """ True if main_argument is a youtube file."""
    if Path(argument).is_file():
        argument_file_content = open(argument).read()
        return (
            "youtu" in argument_file_content
            and argument_file_content.startswith("http")
        )
    else:
        return False


def parse_main_argument(argument, export_folder):
    """Function parsing the main_argument argument.
    Returns a dataframe containing the search terms (or the urls if main_argument is a youtube file."""
    # File or string
    if Path(argument).is_file():
        is_file = True
        argument_file_content = open(argument).read()
        # File of urls or search terms
        is_spotify = (
            "spotify" in argument_file_content
            and argument_file_content.startswith("http")
        )
        is_deezer = (
            "deezer" in argument_file_content
            and argument_file_content.startswith("http")
        )
        is_youtube = (
            "youtu" in argument_file_content
            and argument_file_content.startswith("http")
        )
    else:
        is_file = False
        is_spotify = "spotify" in argument
        is_deezer = "deezer" in argument
        # would be equivalent to argument youtube_url, doesn't exist
        is_youtube = False
    if is_spotify:
        if is_file:
            terms = extract_terms_from_file(argument)
            df = get_spotify_songs(terms)
            logger.info(
                "Reading file containing spotify urls at %s.", argument
            )
        else:
            terms = extract_terms_from_arg(argument)
            df = get_spotify_songs(terms)
            logger.info("Reading spotify urls %s.", argument)
    elif is_deezer:
        if is_file:
            terms = extract_terms_from_file(argument)
            df = get_deezer_songs(terms)
            logger.info("Reading file containing deezer urls at %s.", argument)
        else:
            terms = extract_terms_from_arg(argument)
            df = get_deezer_songs(terms)
            logger.info("Reading deezer urls %s.", argument)
    elif is_youtube:
        if is_file:
            df = pd.read_csv(argument, sep="\t", header=None, names=["url"])
            logger.info(
                "Reading file containing youtube urls at %s.", argument
            )
    else:
        if is_file:
            df = pd.read_csv(argument, sep="\t", header=None, names=["title"])
            logger.info(
                "Reading file containing search terms at %s.", argument
            )
        else:
            df = pd.DataFrame(
                [x.strip() for x in argument.split(",")], columns=["title"]
            )
            logger.info("Reading search terms %s.", argument)
    return df


def parse_arguments(args, export_folder):
    """Parse the arguments. Returns a dataframe."""
    if args.spotify_url:
        terms = extract_terms_from_arg(args.spotify_url)
        df = get_spotify_songs(terms)
        logger.info("Reading spotify urls %s.", args.spotify_url)
    elif args.spotify_file:
        terms = extract_terms_from_file(args.spotify_file)
        df = get_spotify_songs(terms)
        logger.info(
            "Reading file containing spotify urls at %s.", args.spotify_file
        )
    elif args.deezer_url:
        terms = extract_terms_from_arg(args.deezer_url)
        df = get_deezer_songs(terms)
        logger.info("Reading deezer urls %s.", args.deezer_url)
    elif args.deezer_file:
        terms = extract_terms_from_file(args.deezer_file)
        df = get_deezer_songs(terms)
        logger.info(
            "Reading file containing deezer urls at %s.", args.deezer_file
        )
    elif args.youtube_file:
        df = pd.read_csv(
            args.youtube_file, sep="\t", header=None, names=["url"]
        )
        logger.info(
            "Reading file containing youtube urls at %s.", args.youtube_file
        )
    elif args.file_name:
        df = pd.read_csv(
            args.file_name, sep="\t", header=None, names=["title"]
        )
        logger.info(
            "Reading file containing search terms at %s.", args.file_name
        )
    return df


def thread_download(df, num_threads, only_audio, export_folder):
    """ Function spawning threads to download a part of the youtube urls. """
    if only_audio:
        logger.info("Downloading audio files with %s threads.", num_threads)
    else:
        logger.info("Downloading video files with %s threads.", num_threads)

    threads = []
    original_folder = os.getcwd()
    Path(export_folder).mkdir(parents=True, exist_ok=True)
    os.chdir(export_folder)

    # youtube urls has to be in column url of df
    list_df = np.array_split(df, num_threads)
    for index, df in enumerate(list_df):
        # Create and start threads.
        t = YdlDownloadThread(index, df, only_audio)
        t.start()
        threads.append(t)

    # Join all threads.
    for t in threads:
        t.join()
    os.chdir(original_folder)


def main():  # pragma: no cover
    args = parse_args()

    # Export folder
    export_folder = args.export_folder_name
    logger.info("Export folder : %s.", export_folder)
    Path(export_folder).mkdir(parents=True, exist_ok=True)

    is_youtube = False
    # Parse main argument
    if args.main_argument:
        df = parse_main_argument(args.main_argument, export_folder)
        is_youtube = main_argument_is_youtube(args.main_argument)
        logger.debug(
            "main_argument : %s, is_youtube : %s.",
            args.main_argument,
            is_youtube,
        )
    # Verify other arguments
    elif not any(
        [
            args.spotify_url,
            args.deezer_url,
            args.file_name,
            args.spotify_file,
            args.deezer_file,
            args.youtube_file,
        ]
    ):
        logger.error("No input. Use the -h flag to see help. Exiting.")
        raise SystemExit()
    # Parse other arguments
    else:
        df = parse_arguments(args, export_folder)

    # Parse download arguments
    if not args.download_video and not args.download_audio:
        logger.info(
            "No download options selected, but the files containing the tracklist and the extracted youtube urls will still be exported. Use the flags -a to download audio, -v to download video."
        )

    # Special mode (flag --no_search_youtube) where the tracklist is exported and no downloads are made (useful to export spotify/deezer playlists as text)
    if args.no_search_youtube:
        logger.info(
            "no_search_youtube mode. Writing tracklist.csv and exiting."
        )
        df.to_csv(
            export_folder + "/tracklist.csv",
            index=False,
            sep="\t",
            header=True,
        )
        raise SystemExit

    df_previous = None
    # Check if the export folder already contains tracklist.csv from a previous export.
    if Path(export_folder + "/tracklist.csv").is_file():
        logger.info("Detected previous export. Loading file tracklist.txt.")
        df_previous = pd.read_csv(export_folder + "/tracklist.csv", sep="\t",)
        logger.debug(df_previous)

    # Extract youtube urls except if df already contains youtube urls.
    if not is_youtube and not args.youtube_file:
        logger.info("Extracting youtube urls.")
        list_urls = []
        # loop through the df, search youtube urls for each row
        for _, x in tqdm(df.iterrows(), dynamic_ncols=True, total=df.shape[0]):
            # if x["title"] is in df_previous, no need to search on youtube
            if (
                df_previous is not None
                and x["title"] in df_previous["title"].values
            ):
                url = (
                    df_previous[df_previous["title"] == x["title"]]["url"]
                    .to_string(index=False, header=False)
                    .strip()
                )
                logger.debug(
                    "url already in tracklist.csv for %s : %s.",
                    x["title"],
                    url,
                )
                list_urls.append(url)
            else:
                list_urls.append(get_youtube_url(x["title"]))

        with open(
            export_folder + "/urls_list.csv", "w", encoding="utf-8"
        ) as f:
            for i in list_urls:
                f.write(f"{i}\n")

        df["url"] = pd.Series(list_urls).values
        df.to_csv(
            export_folder + "/tracklist.csv",
            index=False,
            sep="\t",
            header=True,
        )
    # else:
    #     # Transform df containing youtube urls to be compatible with ydl_download function
    #     df.columns = ["url"]

    if args.download_audio:
        only_audio = True
        export_folder_audio = export_folder + "/Audio"
        thread_download(df, args.num_threads, only_audio, export_folder_audio)
    if args.download_video:
        only_audio = False
        export_folder_video = export_folder + "/Video"
        thread_download(df, args.num_threads, only_audio, export_folder_video)

    logger.info("Runtime : %.2f seconds." % (time.time() - temps_debut))


def parse_args():  # pragma: no cover
    parser = argparse.ArgumentParser(
        description="Convert spotify/deezer/text albums/playlists to youtube urls or audio/video files."
    )
    parser.add_argument(
        "main_argument",
        nargs="?",
        type=str,
        help="Any search terms allowed : search terms (quoted and separated by comma), deezer/spotify playlist/album urls (separated by comma) or filename containing search terms : deezer/spotify album/playlist urls (one by line) or youtube urls (one by line).",
    )
    parser.add_argument(
        "--debug",
        help="Display debugging information.",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.INFO,
    )
    parser.add_argument(
        "-f",
        "--file_name",
        type=str,
        help="File containing the name of the songs (one search term by line).",
    )
    parser.add_argument(
        "-su",
        "--spotify_url",
        type=str,
        help="Url of the spotify album/playlist urls (separated by comma).",
    )
    parser.add_argument(
        "-du",
        "--deezer_url",
        type=str,
        help="Url of the deezer album/playlist urls (separated by comma).",
    )
    parser.add_argument(
        "-sf",
        "--spotify_file",
        type=str,
        help="File containing the links of the spotify album/playlist urls (one by line).",
    )
    parser.add_argument(
        "-df",
        "--deezer_file",
        type=str,
        help="File containing the links of the deezer album/playlist urls (one by line).",
    )
    parser.add_argument(
        "-yf",
        "--youtube_file",
        type=str,
        help="File containing youtube urls (one by line). The file urls_list.csv exported by ypc is a good candidate.",
    )
    parser.add_argument(
        "-n",
        "--export_folder_name",
        type=str,
        help="Name of the export. Used to name the exports folder.",
        default="ypc_exports",
    )
    parser.add_argument(
        "-v",
        "--download_video",
        help="Download the videos of the tracks found.",
        dest="download_video",
        action="store_true",
    )
    parser.add_argument(
        "-a",
        "--download_audio",
        help="Download the audio files of the tracks found.",
        dest="download_audio",
        action="store_true",
    )
    parser.add_argument(
        "--no_search_youtube",
        help="Doesn't search youtube urls. Use it with the -su/-du/-sf/-df flags if you want to export only the track names from the albums/playlists.",
        dest="no_search_youtube",
        action="store_true",
    )
    parser.add_argument(
        "--num_threads",
        help="Number of threads to use to download the audio/video files (Default: 4, only effective if the -a/-v flags are set).",
        type=int,
        default=4,
    )
    parser.set_defaults(
        download_video=False, download_audio=False, no_search_youtube=False
    )
    args = parser.parse_args()

    logging.basicConfig(level=args.loglevel, format=FORMAT)
    return args


if __name__ == "__main__":
    main()



from ypc import __main__ as ypc
import pandas as pd
import argparse
import pytest
import os

DEEZER_ALBUM_URL = "https://www.deezer.com/fr/album/95467"
DEEZER_FILE = "tests/test_files/deezer_urls.txt"
DEEZER_PLAYLIST_URL = "https://www.deezer.com/fr/playlist/1914526462"
SEARCH_FILE = "tests/test_files/search_terms.txt"
SEARCH_TERM = "xtc - making plans for nigel"
SPOTIFY_ALBUM_URL = "https://open.spotify.com/album/4FCoFSNIFhK36holxHWCnc"
SPOTIFY_FILE = "tests/test_files/spotify_urls.txt"
SPOTIFY_PLAYLIST_URL = (
    "https://open.spotify.com/playlist/37i9dQZF1DX2sUQwD7tbmL"
)
YOUTUBE_FILE = "tests/test_files/youtube_urls.txt"
YOUTUBE_URL = "https://www.youtube.com/watch?v=WfKhVV-7lxI"


def test_extract_terms_from_file():
    if ypc.extract_terms_from_file("tests/test_files/search_terms.txt") != [
        "xtc - making plans for nigel",
        "can - vitamin c",
        "david bowie - heroes",
    ]:
        raise AssertionError()

    with pytest.raises(Exception):
        ypc.extract_terms_from_file("tests/test_files/invalid.txt")


def test_extract_terms_from_arg():
    if ypc.extract_terms_from_arg("term1, term2,term3") != [
        "term1",
        "term2",
        "term3",
    ]:
        raise AssertionError()


def test_main_argument_is_youtube():
    if ypc.main_argument_is_youtube(SEARCH_FILE):
        raise AssertionError()
    if ypc.main_argument_is_youtube(DEEZER_FILE):
        raise AssertionError()
    if ypc.main_argument_is_youtube(SPOTIFY_FILE):
        raise AssertionError()
    if not ypc.main_argument_is_youtube(YOUTUBE_FILE):
        raise AssertionError()
    if ypc.main_argument_is_youtube("test"):
        raise AssertionError()


def test_parse_argument():
    args_search = argparse.Namespace(
        spotify_url=None,
        spotify_file=None,
        deezer_url=None,
        deezer_file=None,
        youtube_file=None,
        file_name=SEARCH_FILE,
    )
    args_deezer_file = argparse.Namespace(
        spotify_url=None,
        spotify_file=None,
        deezer_url=None,
        deezer_file=DEEZER_FILE,
        youtube_file=None,
        file_name=None,
    )
    args_deezer_url = argparse.Namespace(
        spotify_url=None,
        spotify_file=None,
        deezer_url=DEEZER_ALBUM_URL,
        deezer_file=None,
        youtube_file=None,
        file_name=None,
    )
    args_spotify_file = argparse.Namespace(
        spotify_url=None,
        spotify_file=SPOTIFY_FILE,
        deezer_url=None,
        deezer_file=None,
        youtube_file=None,
        file_name=None,
    )
    args_spotify_url = argparse.Namespace(
        spotify_url=SPOTIFY_ALBUM_URL,
        spotify_file=None,
        deezer_url=None,
        deezer_file=None,
        youtube_file=None,
        file_name=None,
    )
    args_youtube = argparse.Namespace(
        spotify_url=None,
        spotify_file=None,
        deezer_url=None,
        deezer_file=None,
        youtube_file=YOUTUBE_FILE,
        file_name=None,
    )
    args_no_youtube = argparse.Namespace(
        spotify_url=None,
        spotify_file=None,
        deezer_url=None,
        deezer_file=None,
        youtube_file="https://youtube.com",
        file_name=None,
    )
    if not isinstance(
        ypc.parse_arguments(args_search, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_arguments(args_deezer_file, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_arguments(args_deezer_url, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_arguments(args_spotify_file, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_arguments(args_spotify_url, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_arguments(args_youtube, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()

    with pytest.raises(Exception):
        ypc.parse_arguments("invalid", "invalid")
        ypc.parse_arguments(args_no_youtube, "ypc_export"), pd.DataFrame


def test_parse_main_argument():
    if not isinstance(
        ypc.parse_main_argument(DEEZER_ALBUM_URL, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_main_argument(DEEZER_FILE, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_main_argument(DEEZER_PLAYLIST_URL, "ypc_export"),
        pd.DataFrame,
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_main_argument(SEARCH_FILE, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_main_argument(SEARCH_TERM, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_main_argument(SPOTIFY_ALBUM_URL, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_main_argument(SPOTIFY_FILE, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_main_argument(SPOTIFY_PLAYLIST_URL, "ypc_export"),
        pd.DataFrame,
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_main_argument(YOUTUBE_FILE, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()
    if not isinstance(
        ypc.parse_main_argument(YOUTUBE_URL, "ypc_export"), pd.DataFrame
    ):
        raise AssertionError()


@pytest.mark.skipif(
    "TRAVIS" in os.environ and os.environ["TRAVIS"] == "true",
    reason="doesn't work with Travis",
)
def test_parse_args():
    if not isinstance(ypc.parse_args(), argparse.Namespace):
        raise AssertionError()
